#include "mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    menuBar = new QMenuBar();

    //3.2.1 File Menu
    fileMenu = new QMenu("&File");

    exitAction = new QAction("E&xit",fileMenu);
//  exitAction->setShortcut(QKeySequence::Quit);//original code from assignment. Ctrl+q is not supported on windows.
    exitAction->setShortcut(QKeySequence(tr("Ctrl+Q")));//workaround for windows.
    connect(exitAction , SIGNAL(triggered ( ) ) ,    this , SLOT(close ( ) ) ) ;

    fileMenu->addAction(exitAction );
    menuBar->addMenu(fileMenu );


    setMenuBar(menuBar);

    //3.2.3 About Message Box
    aboutAction = new QAction("About",menuBar);
    connect(aboutAction, SIGNAL(triggered()), this,SLOT(showAboutBox()));
    menuBar->addAction(aboutAction);

    //3.3 Toolbar
    toolBar = new QToolBar("ToolBar",this);
    addToolBar( toolBar);

    //3.4 Status Bar
    statusBar = new QStatusBar(this);
    statusBar->showMessage("test status message");
    setStatusBar(statusBar);

    //4.0 Widget //Assignment 2: Multiple Widgets.
    qDebug("widgets...");
    {
    controller = new Controller(statusBar);
    myGLWidget = new MyGLWidget(0,controller, true, QQuaternion(),this);
    setCentralWidget(myGLWidget);

    viewFront = new MyGLWidget(1,controller, false, QQuaternion(),this);
    viewLeft = new MyGLWidget(2,controller, false, QQuaternion::fromAxisAndAngle(0,1,0,90),this);//TODO check rotations.
    viewTop = new MyGLWidget(3,controller, false, QQuaternion::fromAxisAndAngle(1,0,0,90),this);

    viewSplitter1 = new QSplitter;
    viewSplitter1->addWidget(myGLWidget);
    viewSplitter1->addWidget(viewFront);

    viewSplitter2 = new QSplitter;
    viewSplitter2->addWidget(viewLeft);
    viewSplitter2->addWidget(viewTop);

    viewSplitter3 = new QSplitter;
    viewSplitter3->setOrientation(Qt::Vertical);
    viewSplitter3->addWidget(viewSplitter1);
    viewSplitter3->addWidget(viewSplitter2);

    viewLayout = new QGridLayout(this);
    viewLayout->addWidget(viewSplitter3);

    QWidget* central_widget = new QWidget(this);
    central_widget->setLayout(viewLayout);
    setCentralWidget(central_widget);
}
    qDebug("tesselation...");

    //tesselation slider
    tesselationSlider = new QSlider(Qt::Horizontal,this);
    tesselationSlider->setTickPosition(QSlider::TicksBelow);
    tesselationSlider->setTickInterval(2);
    tesselationSlider->setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    tesselationSlider->setMinimum(0);
    tesselationSlider->setMaximum(10);
    tesselationSlider->setValue(1);
    toolBar->addWidget(tesselationSlider);
    connect(tesselationSlider,SIGNAL(valueChanged(int)),controller,SLOT(setTessellation(int)));

    //4.1.1 connect shading actions to slot
    MyGLWidget* allWidgets[4] = {myGLWidget, viewFront, viewLeft, viewTop};
    controller->initViewWidgets(allWidgets,viewSplitter1,viewSplitter3,viewLayout);


    //reset Camera button
    resetCameraAction = new QAction("Reset Camera",toolBar);
    resetCameraAction->setIcon(QIcon(":/img/cam_home.png"));
    connect(resetCameraAction,SIGNAL(triggered(bool)),controller,SLOT(resetCamera()));
    toolBar->addAction(resetCameraAction);
    toolBar->insertSeparator(resetCameraAction);


    /* Assignment 2*/
    qDebug("Camera / Object Manipulation...");
    //2 Camera Mode, Object Manipulation Mode
    modeCameraAction = new QAction("Camera Mode",toolBar);
    modeCameraAction->setIcon(QIcon(":/grapa-a2-iconset/camera.png"));
    modeCameraAction->setCheckable(true);
    modeCameraAction->setChecked(true);
    //TODO connect slot
    toolBar->addAction(modeCameraAction);
    toolBar->insertSeparator(modeCameraAction);
    connect(modeCameraAction,SIGNAL(triggered(bool)),controller,SLOT(switchToModeCamera()));

    modeManipulateAction = new QAction("Manipulation Mode", toolBar);
    modeManipulateAction->setIcon(QIcon(":/grapa-a2-iconset/select.png"));
    modeManipulateAction->setCheckable(true);
    //TODO connect slot
    toolBar->addAction(modeManipulateAction);
    connect(modeManipulateAction,SIGNAL(triggered(bool)),controller,SLOT(switchToModeManipulate()));

    controller->initActions(modeCameraAction, modeManipulateAction);

    modeSwitchGroup = new QActionGroup(toolBar);
    modeSwitchGroup->addAction(modeCameraAction);
    modeSwitchGroup->addAction(modeManipulateAction);


    qDebug("View switch...");

    //2 Switch view modes
    viewButton = new QToolButton(toolBar);
    viewButton->setIcon(QIcon(":/grapa-a2-iconset/viewports.png"));
    viewMenu = new QMenu(viewButton);
    viewButton->setMenu(viewMenu);
    viewButton->setPopupMode(QToolButton::MenuButtonPopup);
    toolBar->addWidget(viewButton);
    viewGroup = new QActionGroup(viewMenu);

    singleViewAction = new QAction("Single View",viewMenu);
    singleViewAction->setIcon(QIcon(":/grapa-a2-iconset/view-single.png"));
    singleViewAction->setCheckable(true);
    singleViewAction->setChecked(true);
    singleViewAction->setShortcut(QKeySequence(Qt::Key_1));
    connect(singleViewAction,SIGNAL(triggered(bool)),controller, SLOT(showSingleView()));
    viewMenu->addAction(singleViewAction);
    viewMenu->setDefaultAction(singleViewAction);
    viewGroup->addAction(singleViewAction);

    dualViewAction = new QAction("Dual View",viewMenu);
    dualViewAction->setIcon(QIcon(":/grapa-a2-iconset/view-dual.png"));
    dualViewAction->setCheckable(true);
    dualViewAction->setShortcut(QKeySequence(Qt::Key_2));
    connect(dualViewAction,SIGNAL(triggered(bool)),controller, SLOT(showDoubleView()));
    viewMenu->addAction(dualViewAction);
    viewGroup->addAction(dualViewAction);

    quadViewAction = new QAction("Quad View",viewMenu);
    quadViewAction->setIcon(QIcon(":/grapa-a2-iconset/viewports.png"));
    quadViewAction->setCheckable(true);
    quadViewAction->setShortcut(QKeySequence(Qt::Key_4));
    connect(quadViewAction,SIGNAL(triggered(bool)),controller, SLOT(showQuadView()));
    viewMenu->addAction(quadViewAction);
    viewGroup->addAction(quadViewAction);

    controller->showSingleView();//for whatever reason this works better than setting everything up above. After spending 2 hours setting up the views I honestly don*t care anymore about coding this prettier.

    //Assignment 2: 3) add boxes etc.
{
    addSphere = new QAction("Add Sphere", toolBar);
    addBox = new QAction("Add Box", toolBar);
    addCylinder = new QAction("Add Cylinder", toolBar);
    addCone = new QAction("Add Cone", toolBar);
    addTorus = new QAction("Add Torus", toolBar);
    deletePrimitive = new QAction("Delete", toolBar);

    addSphere->setIcon(QIcon(":/grapa-a2-iconset/sphere.png"));
    addBox->setIcon(QIcon(":/grapa-a2-iconset/box.png"));
    addCylinder->setIcon(QIcon(":/grapa-a2-iconset/cylinder.png"));
    addCone->setIcon(QIcon(":/grapa-a2-iconset/cone.png"));
    addTorus->setIcon(QIcon(":/grapa-a2-iconset/torus.png"));
    deletePrimitive->setIcon(QIcon(":/grapa-a2-iconset/delete.png"));

    addSphere->setCheckable(false);
    addBox->setCheckable(false);
    addCylinder->setCheckable(false);
    addCone->setCheckable(false);
    addTorus->setCheckable(false);
    deletePrimitive->setCheckable(false);

    toolBar->addAction(addSphere);
    toolBar->insertSeparator(addSphere);
    toolBar->addAction(addBox);
    toolBar->addAction(addCylinder);
    toolBar->addAction(addCone);
    toolBar->addAction(addTorus);
    toolBar->addAction(deletePrimitive);

    connect(addSphere, SIGNAL(triggered(bool)),controller, SLOT(addSphere()));
    connect(addBox, SIGNAL(triggered(bool)),controller, SLOT(addBox()));
    connect(addCylinder, SIGNAL(triggered(bool)),controller, SLOT(addCylinder()));
    connect(addCone, SIGNAL(triggered(bool)),controller, SLOT(addCone()));
    connect(addTorus, SIGNAL(triggered(bool)),controller, SLOT(addTorus()));
    connect(deletePrimitive, SIGNAL(triggered(bool)),controller, SLOT(deletePrimitive()));

    deletePrimitive->setShortcut(QKeySequence(Qt::Key_Delete));


    dockWidget = new QDockWidget(tr("Scenegraph"),this);
    dockWidget->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);

    treeView = new QTreeView(this);
    treeView->setModel(controller->getSceneGraph()->getModel());
    treeView->expandAll();

    dockWidget->setWidget(treeView);
    addDockWidget(Qt::RightDockWidgetArea, dockWidget);


    connect(treeView,SIGNAL(clicked(QModelIndex)),controller, SLOT(itemSelected(QModelIndex)));
    connect(treeView,SIGNAL(doubleClicked(QModelIndex)),controller,SLOT(itemDoubleClicked(QModelIndex)));
    connect(treeView, SIGNAL(doubleClicked(QModelIndex)),this,SLOT(showTextEditBox()));

    controller->initTreeView(treeView);

//    textInput = new QLineEdit(toolBar);
//    textInput->setText("Test");
//    toolBar->addWidget(textInput);

    }



    qDebug("Done initializing");


}

MainWindow::~MainWindow()
{

}

//3.2.3 about message box
void MainWindow::showAboutBox(){
    QMessageBox msgBox;
    msgBox.setWindowTitle("About Hello Cube!");
    msgBox.setText("Written by Alisa");
    msgBox.exec();
}

void MainWindow::showTextEditBox(){
    bool ok;
    QString text = QInputDialog::getText(this, "Edit Name",
                                         tr("New Name:"), QLineEdit::Normal,
                                         controller->getCurrentName(), &ok);
    if (ok && !text.isEmpty()){
        qDebug("Text received");
        controller->changeItemName(text);
    }
}
