#include "scenegraph.h"
#include "myglwidget.h"
#include "primitive.h"

SceneGraph::SceneGraph()
{
    nodes = QList<RigidBodyTransformation*>();
    views = QList<MyGLWidget*>();
    idCounter = 0;
    sphereIndex = 1;
    boxIndex = 1;
    cylIndex = 1;
    coneIndex = 1;
    torusIndex = 1;

    model = new MyItemModel;
}

//Scenegraph gets a list of all views it needs to notify on changes
void SceneGraph::registerView(MyGLWidget* view){
    views.append(view);
}

MyItemModel* SceneGraph::getModel(){
    return model;
}

//add primitives to sceneGraph
RigidBodyTransformation* SceneGraph::addSphere(int tesselation){
    nextSphereName = "Sphere ";
    nextSphereName.append(QString::number(sphereIndex));
    //tesselation < 3 makes no sense (except for box)
    int t = tesselation;
    if (t < 3) t = 3;
    Primitive* p = new Primitive(nextSphereName, idCounter, t, Primitive::Type::SPHERE);
    sphereIndex++;

    qDebug() << "New Sphere added in scenegraph." << nextSphereName;

    return addPrimitive(p);
}

RigidBodyTransformation* SceneGraph::addBox(int tesselation){

    nextBoxName = "Box ";
    nextBoxName.append(QString::number(boxIndex));
    Primitive* p = new Primitive(nextBoxName, idCounter, tesselation, Primitive::Type::BOX);
    boxIndex++;

    qDebug("New box added in scenegraph");

    return addPrimitive(p);
}

RigidBodyTransformation* SceneGraph::addCylinder(int tesselation){
    nextCylName = "Cylinder ";
    nextCylName.append(QString::number(cylIndex));
    int t = tesselation;
    if (t < 3) t = 3;
    Primitive* p = new Primitive(nextCylName, idCounter, t, Primitive::Type::CYLINDER);
    cylIndex++;
    qDebug("added Cylinder to scenegraph");
    return addPrimitive(p);
}

RigidBodyTransformation* SceneGraph::addCone(int tesselation){

    nextConeName = "Cone ";
    nextConeName.append(QString::number(coneIndex));
    int t = tesselation;
    if (t < 3) t = 3;
    Primitive* p = new Primitive(nextConeName, idCounter, t, Primitive::Type::CONE);
    coneIndex++;
    return addPrimitive(p);
}

RigidBodyTransformation* SceneGraph::addTorus(int tesselation){

    nextTorusName = "Torus ";
    nextTorusName.append(QString::number(torusIndex));
    int t = tesselation;
    if (t < 3) t = 3;
    Primitive* p = new Primitive(nextTorusName, idCounter, t, Primitive::Type::TORUS);
    torusIndex++;
    return addPrimitive(p);
}

RigidBodyTransformation* SceneGraph::addPrimitive(Primitive* p){
    RigidBodyTransformation* r = new RigidBodyTransformation();
    QString n = "RBT Group ";
    n.append(QString::number(idCounter));
    idCounter++;
    r->setName(n);
    nodes.append(r);
    lastAddedIndex = model->addNewTransformToRoot(r,p);
    qDebug() << "added primitive to root. childlength: " << r->getChildren().length();
//    model->insertRows(1,1,QModelIndex());
    notifyViews();
    return r;
}

RigidBodyTransformation* SceneGraph::deleteNode(RigidBodyTransformation *r){
    if (r == 0){
        qDebug("transform already 0");
        return 0;
    }
    bool s = nodes.removeOne(r);
    if (!s) qDebug("could not remove this. Delete r anyways.");
    model->removeTransformFromRoot(r);
    r->~RigidBodyTransformation();
    notifyViews();
    if (nodes.length() > 0) return nodes.back();
    else return 0;
}

QModelIndex SceneGraph::getIndex(TreeItem *item){
    return model->getIndex(item);
}

QModelIndex SceneGraph::getLastAddedIndex(){
    qDebug() << "last index " << lastAddedIndex;
    return lastAddedIndex;
}

//transform something
void SceneGraph::addRotation(RigidBodyTransformation *r, QQuaternion diff){
    r->addRotation(diff);
    notifyViews();
}

void SceneGraph::addTranslation(RigidBodyTransformation *r, QVector3D diff){
    r->addTranslation(diff);
    notifyViews();
}

//for drawing
const QList<RigidBodyTransformation*> SceneGraph::getGraph(){
    return nodes;
}

//notifies views whenever something changes (is added or transformed)
void SceneGraph::notifyViews(){
    QList<MyGLWidget*>::iterator i;
    for (i = views.begin(); i != views.end(); i++){
        (*i)->modelChanged();
    }
}

TreeItem* SceneGraph::getItem(QModelIndex q){
    return model->getItem(q);
}
