#ifndef SCENEGRAPH_H
#define SCENEGRAPH_H

#include <QList>
#include <rigidbodytransformation.h>
#include <QString>
#include <myitemmodel.h>

class MyGLWidget;

class SceneGraph
{
public:
    SceneGraph();

    RigidBodyTransformation* addSphere(int tesselation);
    RigidBodyTransformation* addBox(int tesselation);
    RigidBodyTransformation* addCylinder(int tesselation);
    RigidBodyTransformation* addCone(int tesselation);
    RigidBodyTransformation* addTorus(int tesselation);

    void drawAll();
    void registerView(MyGLWidget* view);
    void notifyViews();

    void addTranslation(RigidBodyTransformation* r, QVector3D diff);
    void addRotation(RigidBodyTransformation* r, QQuaternion diff);

    const QList<RigidBodyTransformation*> getGraph();

    //automatically selects last added primitive. if none left, selects 0.
    RigidBodyTransformation* deleteNode(RigidBodyTransformation* r);


    MyItemModel* getModel();

    TreeItem* getItem(QModelIndex q);
    QModelIndex getIndex(TreeItem* item);
    QModelIndex getLastAddedIndex();//Index of last added tree item

//Hallo. Todos:
    //Log selected name in status bar
    //add remaining primitive types
    //extract draw method to something more view-like
    //delete primitives
private:
    int idCounter;

    int sphereIndex;
    int boxIndex;
    int cylIndex;
    int coneIndex;
    int torusIndex;

    QString nextSphereName;
    QString nextBoxName;
    QString nextCylName;
    QString nextConeName;
    QString nextTorusName;

    RigidBodyTransformation* addPrimitive(Primitive* p);

    QList<RigidBodyTransformation*> nodes;
    QList<MyGLWidget*> views;


    MyItemModel* model;

    QModelIndex lastAddedIndex;


};

#endif // SCENEGRAPH_H
